@@ -1,13 +1,12 @@
 import abc
 import logging as log
-from os import path
 from past.builtins import long
 import requests
 from requests import exceptions
 import six
 import xmltodict
 
-log.basicConfig(level=log.DEBUG)
+#log.basicConfig(level=log.DEBUG)
 log = log.getLogger(__name__)
 
 
@@ -16,6 +15,12 @@ class HttpClient(object):
 
     endpoint = None
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        pass
+
     def _do_request(self, http_method, rest_command,
                     parameters=None,
                     file=None):
@@ -42,29 +47,28 @@ class HttpClient(object):
                 response = requests.request(http_method, cmd_url,
                                             params=parameters,
                                             verify=False)
-            if response.status_code > 299:
-                raise exceptions.HTTPError
+            if 400 < response.status_code < 500:
+                raise KempTechApiError(code=response.status_code)
+            else:
+                response.raise_for_status()
         except exceptions.ConnectionError:
-            log.exception("A connection error occurred to %s",
-                          self.endpoint)
-            raise KempTechApiError(response.text)
+            log.exception("A connection error occurred to %s.", self.endpoint)
+            raise
         except exceptions.URLRequired:
-            log.exception("%s is not a valid URL to make a request with.",
-                          cmd_url)
-            raise KempTechApiError(response.text)
+            log.exception("%s is an invalid URL", cmd_url)
+            raise
         except exceptions.TooManyRedirects:
-            log.exception("Too many redirects with request to %s", cmd_url)
-            raise KempTechApiError(response.text)
+            log.exception("Too many redirects with request to %s.", cmd_url)
+            raise
         except exceptions.Timeout:
-            log.exception("A connection error occurred to %s", self.endpoint)
-            raise KempTechApiError(response.text)
+            log.exception("A connection %s has timed out.", self.endpoint)
+            raise
         except exceptions.HTTPError:
-            log.exception("A HTTP error occurred with request to %s", cmd_url)
-            raise KempTechApiError(response.text)
+            log.exception("A HTTP error occurred with request to %s.", cmd_url)
+            raise KempTechApiError(msg=response.text)
         except exceptions.RequestException:
-            log.exception("An unknown error occurred with request to %s",
-                          cmd_url)
-            raise KempTechApiError(response.text)
+            log.exception("An error occurred with request to %s.", cmd_url)
+            raise
         return response.text
 
     def _get(self, rest_command, parameters):
@@ -160,11 +164,27 @@ class LoadMaster(HttpClient):
         response = self._get('get', parameters)
         value = ApiXmlHelper.get_data_field(response, parameter)
         if isinstance(value, dict):
-            # This converts HTML to an awful one string disaster.
+            # This hack converts possible HTML to an awful one string
+            # disaster instead of returning parsed html as an OrderedDict.
             value = "".join("{!s}={!r}".format(key, val) for (key, val) in
                             value.items())
         return value
 
+    def enable_api(self):
+        """Enable LoadMaster API"""
+        # Can't use the HttpClient methods for this as the
+        # endpoint is different when attempting to enable the API.
+        url = ("https://{user}:{pw}@{ip}"
+               "/progs/doconfig/enableapi/set/yes/").format(user=self.username,
+                                                            pw=self.password,
+                                                            ip=self.ip_address)
+        try:
+            response = requests.get(url, verify=False)
+        except exceptions.RequestException:
+            log.exception("An unknown error occurred with request to %s",
+                          url)
+            raise KempTechApiError(response.text)
+
     def update_firmware(self, file):
         response = self._post('installpatch', file)
         return ApiXmlHelper.is_successful(response)
@@ -175,18 +195,26 @@ class KempTechApiError(Exception):
 
     def __init__(self, msg=None, code=None):
         if msg is not None:
+            # 400 Errors should be handled here. This will pass the error
+            # message given by the LoadMaster API and show it as the exception
+            # message in the traceback.
             message = ApiXmlHelper.get_error_msg(msg)
         else:
+            # 400+ shouldn't be handled here, but cover it anyway. Should just
+            # re-raise whatever requests throws instead of depending on this.
+            # However, 400 itself should be handled, just not in this block.
+            # 404 is the only truly useful one here so far.
             if code == 400:
-                message = "Mandatory parameter missing from request."
+                message = "400 Mandatory parameter missing from request."
             elif code == 401:
-                message = "Username or password is missing or is incorrect."
+                message = "401 Client Error: Authorization required."
             elif code == 403:
-                message = "Incorrect permissions."
+                message = "403 Incorrect permissions."
             elif code == 404:
-                message = "Not found."
+                message = ("404 Not found."
+                           "Ensure the API is enabled on the LoadMaster.")
             elif code == 405:
-                message = "Unknown command."
+                message = "405 Unknown command."
             else:
                 message = "An unknown error has occurred."
         super(KempTechApiError, self).__init__(message)
@@ -204,11 +232,11 @@ class ApiXmlHelper(object):
 
     @classmethod
     def is_successful(cls, xml):
-        """Return True if request was successful, else return Error msg."""
+        """Return True if xml response contains a success, else false."""
         if ApiXmlHelper.get_success_msg(xml):
             return True
         else:
-            return ApiXmlHelper.get_error_msg(xml)
+            return False
 
     @classmethod
     def get_data_field(cls, xml, field):
